@@ -1,40 +1,42 @@
-from urllib import quote_plus
-
-from zope.interface import implements
-from zope.component import getMultiAdapter, queryMultiAdapter
-from zope.app.component.hooks import getSite
-
-from zope.i18n import translate
-from zope.i18nmessageid.message import Message
-
-from zope.app.publisher.browser.menu import BrowserMenu
-from zope.app.publisher.browser.menu import BrowserSubMenuItem
+from cgi import escape
 
 from plone.memoize.instance import memoize
+from plone.app.content.browser.folderfactories import _allowedTypes
+from plone.app.content.browser.interfaces import IContentsPage
+from zope.interface import implements
+from zope.component import getMultiAdapter, queryMultiAdapter
 
-from Acquisition import aq_inner, aq_base
-
-from Products.CMFCore.utils import getToolByName
+from Acquisition import aq_base
+from Products.CMFCore.utils import getToolByName, _checkPermission
 from Products.CMFDynamicViewFTI.interface import ISelectableBrowserDefault
-
+from Products.CMFPlone import utils
 from Products.CMFPlone.interfaces.structure import INonStructuralFolder
 from Products.CMFPlone.interfaces.constrains import IConstrainTypes
 from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
 
-from interfaces import IActionsSubMenuItem
-from interfaces import IDisplaySubMenuItem
-from interfaces import IFactoriesSubMenuItem
-from interfaces import IWorkflowSubMenuItem
+from plone.app.contentmenu import PloneMessageFactory as _
+from plone.app.contentmenu.interfaces import IActionsMenu
+from plone.app.contentmenu.interfaces import IActionsSubMenuItem
+from plone.app.contentmenu.interfaces import IDisplayMenu
+from plone.app.contentmenu.interfaces import IDisplaySubMenuItem
+from plone.app.contentmenu.interfaces import IFactoriesMenu
+from plone.app.contentmenu.interfaces import IFactoriesSubMenuItem
+from plone.app.contentmenu.interfaces import IWorkflowMenu
+from plone.app.contentmenu.interfaces import IWorkflowSubMenuItem
 
-from interfaces import IActionsMenu
-from interfaces import IDisplayMenu
-from interfaces import IFactoriesMenu
-from interfaces import IWorkflowMenu
+# BBB Zope 2.12
+try:
+    from zope.browsermenu.menu import BrowserMenu
+    from zope.browsermenu.menu import BrowserSubMenuItem
+except ImportError:
+    from zope.app.publisher.browser.menu import BrowserMenu
+    from zope.app.publisher.browser.menu import BrowserSubMenuItem
 
-from Products.CMFPlone import utils
-from Products.CMFPlone import PloneMessageFactory as _
+try:
+    from Products.CMFPlacefulWorkflow import ManageWorkflowPolicies
+except ImportError:
+    from Products.CMFCore.permissions import ManagePortal as ManageWorkflowPolicies
 
-from plone.app.content.browser.folderfactories import _allowedTypes
 
 def _safe_unicode(text):
     if not isinstance(text, unicode):
@@ -56,9 +58,6 @@ class ActionsSubMenuItem(BrowserSubMenuItem):
         BrowserSubMenuItem.__init__(self, context, request)
         self.context_state = getMultiAdapter((context, request), name='plone_context_state')
 
-    def getToolByName(self, tool):
-        return getToolByName(getSite(), tool)
-
     @property
     def action(self):
         folder = self.context
@@ -68,8 +67,10 @@ class ActionsSubMenuItem(BrowserSubMenuItem):
 
     @memoize
     def available(self):
-        actions_tool = self.getToolByName('portal_actions')
-        editActions = actions_tool.listActionInfos(object=aq_inner(self.context), categories=('object_buttons',), max=1)
+        if IContentsPage.providedBy(self.request):
+            return False
+        actions_tool = getToolByName(self.context, 'portal_actions')
+        editActions = actions_tool.listActionInfos(object=self.context, categories=('object_buttons',), max=1)
         return len(editActions) > 0
 
     def selected(self):
@@ -83,30 +84,32 @@ class ActionsMenu(BrowserMenu):
         """Return menu item entries in a TAL-friendly form."""
         results = []
 
-        portal_state = getMultiAdapter((context, request), name='plone_portal_state')
-
-        actions_tool = getToolByName(aq_inner(context), 'portal_actions')
-        editActions = actions_tool.listActionInfos(object=aq_inner(context), categories=('object_buttons',))
-
+        context_state = getMultiAdapter((context, request),
+            name='plone_context_state')
+        editActions = context_state.actions('object_buttons')
         if not editActions:
-            return []
+            return results
 
-        plone_utils = getToolByName(context, 'plone_utils')
-        portal_url = portal_state.portal_url()
+        actionicons = getToolByName(context, 'portal_actionicons')
+        portal_url = getToolByName(context, 'portal_url')()
 
         for action in editActions:
             if action['allowed']:
-                cssClass = 'actionicon-object_buttons-%s' % action['id']
-                icon = plone_utils.getIconFor('object_buttons', action['id'], None)
-                if icon:
-                    icon = '%s/%s' % (portal_url, icon)
+                aid = action['id']
+                cssClass = 'actionicon-object_buttons-%s' % aid
+                icon = action.get('icon', None)
+                if not icon:
+                    # allow fallback to action icons tool
+                    icon = actionicons.queryActionIcon('object_buttons', aid)
+                    if icon:
+                        icon = '%s/%s' % (portal_url, icon)
 
                 results.append({ 'title'       : action['title'],
                                  'description' : '',
                                  'action'      : action['url'],
                                  'selected'    : False,
                                  'icon'        : icon,
-                                 'extra'       : {'id': action['id'], 'separator': None, 'class': cssClass},
+                                 'extra'       : {'id': aid, 'separator': None, 'class': cssClass},
                                  'submenu'     : None,
                                  })
 
@@ -198,6 +201,8 @@ class DisplaySubMenuItem(BrowserSubMenuItem):
 
     @memoize
     def disabled(self):
+        if IContentsPage.providedBy(self.request):
+            return True
         context = self.context
         if self.context_state.is_default_page():
             context = utils.parent(context)
@@ -276,7 +281,7 @@ class DisplayMenu(BrowserMenu):
                                  'action'      : None,
                                  'selected'    : False,
                                  'icon'        : None,
-                                 'extra'       : {'id': '_folderHeader', 'separator': 'actionSeparator', 'class': ''},
+                                 'extra'       : {'id': 'folderHeader', 'separator': 'actionSeparator', 'class': ''},
                                  'submenu'     : None,
                                  })
 
@@ -287,16 +292,16 @@ class DisplayMenu(BrowserMenu):
                                      'action'      : '%s/selectViewTemplate?templateId=%s' % (folderUrl, id,),
                                      'selected'    : False,
                                      'icon'        : None,
-                                     'extra'       : {'id': id, 'separator': None, 'class': ''},
+                                     'extra'       : {'id': 'folder-' + id, 'separator': None, 'class': ''},
                                      'submenu'     : None,
                                      })
             # Display the selected item (i.e. the context)
-            results.append({ 'title'       : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : _safe_unicode(obj.Title())}),
+            results.append({ 'title'       : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : escape(_safe_unicode(obj.Title()))}),
                              'description' : '',
                              'action'      : None,
                              'selected'    : True,
                              'icon'        : None,
-                             'extra'       : {'id': '_folderDefaultPageDisplay', 'separator': 'actionSeparator', 'class': 'actionMenuSelected'},
+                             'extra'       : {'id': 'folderDefaultPageDisplay', 'separator': 'actionSeparator', 'class': 'actionMenuSelected'},
                              'submenu'     : None,
                              })
             # Let the user change the selection
@@ -306,7 +311,7 @@ class DisplayMenu(BrowserMenu):
                                  'action'      : '%s/select_default_page' % (folderUrl,),
                                  'selected'    : False,
                                  'icon'        : None,
-                                 'extra'       : {'id': '_folderChangeDefaultPage', 'separator': 'actionSeparator', 'class': ''},
+                                 'extra'       : {'id': 'folderChangeDefaultPage', 'separator': 'actionSeparator', 'class': ''},
                                  'submenu'     : None,
                                  })
 
@@ -323,7 +328,7 @@ class DisplayMenu(BrowserMenu):
                                  'action'      : None,
                                  'selected'    : False,
                                  'icon'        : None,
-                                 'extra'       : {'id': '_contextHeader', 'separator': 'actionSeparator', 'class': ''},
+                                 'extra'       : {'id': 'contextHeader', 'separator': 'actionSeparator', 'class': ''},
                                  'submenu'     : None,
                                  })
 
@@ -338,12 +343,13 @@ class DisplayMenu(BrowserMenu):
 
             if showLayouts and contextCanSetLayout:
                 for id, title in contextLayouts:
+                    is_selected = (defaultPage is None and id == selected)
                     results.append({ 'title'       : title,
                                      'description' : '',
                                      'action'      : '%s/selectViewTemplate?templateId=%s' % (contextUrl, id,),
-                                     'selected'    : (defaultPage is None and id == selected),
+                                     'selected'    : is_selected,
                                      'icon'        : None,
-                                     'extra'       : {'id': id, 'separator': None, 'class': ''},
+                                     'extra'       : {'id': id, 'separator': None, 'class': is_selected and 'actionMenuSelected' or ''},
                                      'submenu'     : None,
                                      })
 
@@ -357,7 +363,7 @@ class DisplayMenu(BrowserMenu):
                                          'action'      : '%s/select_default_page' % (contextUrl,),
                                          'selected'    : False,
                                          'icon'        : None,
-                                         'extra'       : {'id': '_contextSetDefaultPage', 'separator': 'actionSeparator', 'class': ''},
+                                         'extra'       : {'id': 'contextSetDefaultPage', 'separator': 'actionSeparator', 'class': ''},
                                          'submenu'     : None,
                                          })
                 else:
@@ -369,12 +375,12 @@ class DisplayMenu(BrowserMenu):
                         else:
                             defaultPageTitle = getattr(aq_base(defaultPageObj), 'title', u'')
 
-                    results.append({ 'title'       : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : _safe_unicode(defaultPageTitle)}),
+                    results.append({ 'title'       : _(u'label_item_selected', default=u'Item: ${contentitem}', mapping={'contentitem' : escape(_safe_unicode(defaultPageTitle))}),
                                      'description' : '',
                                      'action'      : None,
                                      'selected'    : True,
                                      'icon'        : None,
-                                     'extra'       : {'id': '_contextDefaultPageDisplay', 'separator': 'actionSeparator', 'class': ''},
+                                     'extra'       : {'id': 'contextDefaultPageDisplay', 'separator': 'actionSeparator', 'class': ''},
                                      'submenu'     : None,
                                      })
                     if contextCanSetDefaultPage:
@@ -383,7 +389,7 @@ class DisplayMenu(BrowserMenu):
                                          'action'      : '%s/select_default_page' % (contextUrl,),
                                          'selected'    : False,
                                          'icon'        : None,
-                                         'extra'       : {'id': '_contextChangeDefaultPage', 'separator': 'actionSeparator', 'class': ''},
+                                         'extra'       : {'id': 'contextChangeDefaultPage', 'separator': 'actionSeparator', 'class': ''},
                                          'submenu'     : None,
                                          })
 
@@ -395,6 +401,8 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
 
     submenuId = 'plone_contentmenu_factory'
     order = 30
+    title = _(u'label_add_new_item', default=u'Add new\u2026')
+    description = _(u'title_add_new_items_inside_item', default=u'Add new items inside this item')
 
     def __init__(self, context, request):
         BrowserSubMenuItem.__init__(self, context, request)
@@ -402,62 +410,11 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
 
     @property
     def extra(self):
-        return {'id': 'plone-contentmenu-factories',
-                'hideChildren': self._hideChildren()}
-
-    @property
-    def title(self):
-        itemsToAdd = self._itemsToAdd()
-        showConstrainOptions = self._showConstrainOptions()
-        if showConstrainOptions or len(itemsToAdd) > 1:
-            return _(u'label_add_new_item', default=u'Add new\u2026')
-        elif len(itemsToAdd) == 1:
-            fti=itemsToAdd[0][1]
-            title = fti.Title()
-            if isinstance(title, Message):
-                title = translate(title, context=self.request)
-            else:
-                title = translate(_safe_unicode(title),
-                                  domain='plone',
-                                  context=self.request)
-            return _(u'label_add_type', default='Add ${type}',
-                     mapping={'type' : title})
-        else:
-            return _(u'label_add_new_item', default=u'Add new\u2026')
-
-    @property
-    def description(self):
-        itemsToAdd = self._itemsToAdd()
-        showConstrainOptions = self._showConstrainOptions()
-        if showConstrainOptions or len(itemsToAdd) > 1:
-            return _(u'title_add_new_items_inside_item', default=u'Add new items inside this item')
-        elif len(itemsToAdd) == 1:
-            return itemsToAdd[0][1].Description()
-        else:
-            return _(u'title_add_new_items_inside_item', default=u'Add new items inside this item')
+        return {'id': 'plone-contentmenu-factories'}
 
     @property
     def action(self):
-        addContext = self._addContext()
-        if self._hideChildren():
-            (addContext, fti) = self._itemsToAdd()[0]
-            baseUrl = addContext.absolute_url()
-            addingview = queryMultiAdapter((addContext, self.request), name='+')
-            if addingview is not None:
-                addview = queryMultiAdapter((addingview, self.request), name=fti.factory)
-                if addview is not None:
-                    return '%s/+/%s' % (baseUrl, fti.factory,)
-            return '%s/createObject?type_name=%s' % (baseUrl, quote_plus(fti.getId()),)
-        else:
-            return '%s/folder_factories' % self.context_state.folder().absolute_url()
-
-    @property
-    def icon(self):
-        if self._hideChildren():
-            fti = self._itemsToAdd()[0][1]
-            return fti.getIcon()
-        else:
-            return None
+        return '%s/folder_factories' % self._addContext().absolute_url()
 
     def available(self):
         itemsToAdd = self._itemsToAdd()
@@ -471,7 +428,10 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
 
     @memoize
     def _addContext(self):
-        return self.context_state.folder()
+        if self.context_state.is_structural_folder():
+            return self.context
+        else:
+            return self.context_state.folder()
 
     @memoize
     def _itemsToAdd(self):
@@ -497,18 +457,11 @@ class FactoriesSubMenuItem(BrowserSubMenuItem):
         constrain = ISelectableConstrainTypes(addContext, None)
         if constrain is None:
             return False
-        elif constrain.canSetConstrainTypes():
+        elif constrain.canSetConstrainTypes() and constrain.getDefaultAddableTypes():
             return True
         elif len(constrain.getLocallyAllowedTypes()) < len(constrain.getImmediatelyAddableTypes()):
             return True
 
-    @memoize
-    def _hideChildren(self):
-        itemsToAdd = self._itemsToAdd()
-        showConstrainOptions = self._showConstrainOptions()
-        return (len(itemsToAdd) == 1 and not showConstrainOptions)
-
-
 class FactoriesMenu(BrowserMenu):
     implements(IFactoriesMenu)
 
@@ -537,20 +490,20 @@ class FactoriesMenu(BrowserMenu):
                              'action'      : url,
                              'selected'    : False,
                              'icon'        : None,
-                             'extra'       : {'id': '_more', 'separator': None, 'class': ''},
+                             'extra'       : {'id': 'more', 'separator': None, 'class': ''},
                              'submenu'     : None,
                             })
 
         constraints = ISelectableConstrainTypes(addContext, None)
         if constraints is not None:
-            if constraints.canSetConstrainTypes():
+            if constraints.canSetConstrainTypes() and constraints.getDefaultAddableTypes():
                 url = '%s/folder_constraintypes_form' % (addContext.absolute_url(),)
                 results.append({'title'       : _(u'folder_add_settings', default=u'Restrictions\u2026'),
                                 'description' : _(u'title_configure_addable_content_types', default=u'Configure which content types can be added here'),
                                 'action'      : url,
                                 'selected'    : False,
                                 'icon'        : None,
-                                'extra'       : {'id': '_settings', 'separator': None, 'class': ''},
+                                'extra'       : {'id': 'settings', 'separator': None, 'class': ''},
                                 'submenu'     : None,
                                 })
 
@@ -608,8 +561,8 @@ class WorkflowSubMenuItem(BrowserSubMenuItem):
 
     @memoize
     def _transitions(self):
-        wf_tool = getToolByName(aq_inner(self.context), 'portal_workflow')
-        return wf_tool.listActionInfos(object=aq_inner(self.context), max=1)
+        wf_tool = getToolByName(self.context, 'portal_workflow')
+        return wf_tool.listActionInfos(object=self.context, max=1)
 
     @memoize
     def _currentStateTitle(self):
@@ -641,15 +594,14 @@ class WorkflowMenu(BrowserMenu):
     def getMenuItems(self, context, request):
         """Return menu item entries in a TAL-friendly form."""
         results = []
-        context = aq_inner(context)
-
-        wf_tool = getToolByName(context, 'portal_workflow')
-        workflowActions = wf_tool.listActionInfos(object=context)
 
         locking_info = queryMultiAdapter((context, request), name='plone_lock_info')
         if locking_info and locking_info.is_locked_for_current_user():
             return []
 
+        wf_tool = getToolByName(context, 'portal_workflow')
+        workflowActions = wf_tool.listActionInfos(object=context)
+
         for action in workflowActions:
             if action['category'] != 'workflow':
                 continue
@@ -691,18 +643,21 @@ class WorkflowMenu(BrowserMenu):
                              'action'       : url + '/content_status_history',
                              'selected'     : False,
                              'icon'         : None,
-                             'extra'        : {'id': '_advanced', 'separator': 'actionSeparator', 'class': 'kssIgnore'},
+                             'extra'        : {'id': 'advanced', 'separator': 'actionSeparator', 'class': 'kssIgnore'},
                              'submenu'      : None,
                             })
 
         if getToolByName(context, 'portal_placeful_workflow', None) is not None:
-            results.append({ 'title'       : _(u'workflow_policy', default=u'Policy...'),
-                             'description' : '',
-                             'action'      : url + '/placeful_workflow_configuration',
-                             'selected'    : False,
-                             'icon'        : None,
-                             'extra'       : {'id': '_policy', 'separator': None, 'class': 'kssIgnore'},
-                             'submenu'     : None,
+            if _checkPermission(ManageWorkflowPolicies, context):
+                results.append({'title': _(u'workflow_policy',
+                                           default=u'Policy...'),
+                                'description': '',
+                                'action': url + '/placeful_workflow_configuration',
+                                'selected': False,
+                                'icon': None,
+                                'extra': {'id': 'policy', 'separator': None,
+                                          'class': 'kssIgnore'},
+                                'submenu': None,
                             })
 
         return results
