@@ -1,35 +1,38 @@
 import unittest
 
+from Testing.ZopeTestCase.placeless import PlacelessSetup
 from Products.PloneTestCase import PloneTestCase as ptc
-ptc.setupPloneSite(extension_profiles=['Products.CMFPlone:testfixture'])
+ptc.setupPloneSite()
 
-from zope.interface import directlyProvides
+from plone.locking.interfaces import ILockable
 from zope.component import getUtility
-
-from zope.app.publisher.interfaces.browser import IBrowserMenu
-from zope.app.testing.placelesssetup import PlacelessSetup
+from zope.interface import directlyProvides
 
 from Products.CMFCore.Expression import Expression
 from Products.CMFCore.utils import getToolByName
-
 from Products.CMFPlone.interfaces import ISelectableConstrainTypes
 from Products.CMFPlone.interfaces import INonStructuralFolder
+from Products.CMFPlone.tests import dummy
+from Products.CMFPlone.utils import _createObjectByType
 
 from plone.app.contentmenu.interfaces import IActionsMenu
 from plone.app.contentmenu.interfaces import IDisplayMenu
 from plone.app.contentmenu.interfaces import IFactoriesMenu
 from plone.app.contentmenu.interfaces import IWorkflowMenu
 
-from plone.locking.interfaces import ILockable
-
-from Products.CMFPlone.utils import _createObjectByType
-from Products.CMFPlone.tests import dummy
+# BBB Zope 2.12
+try:
+    from zope.browsermenu.interfaces import IBrowserMenu
+except ImportError:
+    from zope.app.publisher.interfaces.browser import IBrowserMenu
 
 
 class TestActionsMenu(ptc.PloneTestCase):
 
     def afterSetUp(self):
-        self.menu = getUtility(IBrowserMenu, name='plone_contentmenu_actions', context=self.folder)
+        self.menu = getUtility(
+            IBrowserMenu, name='plone_contentmenu_actions',
+            context=self.folder)
         self.request = self.app.REQUEST
 
     def testActionsMenuImplementsIBrowserMenu(self):
@@ -46,7 +49,9 @@ class TestActionsMenu(ptc.PloneTestCase):
 class TestDisplayMenu(ptc.PloneTestCase):
 
     def afterSetUp(self):
-        self.menu = getUtility(IBrowserMenu, name='plone_contentmenu_display', context=self.folder)
+        self.menu = getUtility(
+            IBrowserMenu, name='plone_contentmenu_display',
+            context=self.folder)
         self.request = self.app.REQUEST
 
     def testActionsMenuImplementsIBrowserMenu(self):
@@ -78,17 +83,20 @@ class TestDisplayMenu(ptc.PloneTestCase):
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
-        self.failUnless('_folderDefaultPageDisplay' in [a['extra']['id'] for a in actions])
+        self.failUnless('folderDefaultPageDisplay' in
+                        [a['extra']['id'] for a in actions])
         self.failIf('document_view' in [a['extra']['id'] for a in actions])
 
-    def testDefaultPageIncludesParentAndItemViewsWhenItemHasMultipleViews(self):
+    def testDefaultPageIncludesParentAndItemViewsWhenItemHasMultipleViews(
+        self):
         fti = self.portal.portal_types['Document']
         documentViews = fti.view_methods + ('base_view',)
-        fti.manage_changeProperties(view_methods = documentViews)
+        fti.manage_changeProperties(view_methods=documentViews)
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
-        self.failUnless('_folderDefaultPageDisplay' in [a['extra']['id'] for a in actions])
+        self.failUnless('folderDefaultPageDisplay' in
+                        [a['extra']['id'] for a in actions])
         self.failUnless('document_view' in [a['extra']['id'] for a in actions])
         self.failUnless('base_view' in [a['extra']['id'] for a in actions])
 
@@ -104,53 +112,59 @@ class TestDisplayMenu(ptc.PloneTestCase):
         self.folder.invokeFactory('Folder', 'f1')
         self.failUnless(self.folder.f1.canSetDefaultPage())
         actions = self.menu.getMenuItems(self.folder.f1, self.request)
-        self.failUnless('_contextSetDefaultPage' in [a['extra']['id'] for a in actions])
+        self.failUnless('contextSetDefaultPage' in
+                        [a['extra']['id'] for a in actions])
 
     def testWithCanSetDefaultPageFalse(self):
         self.folder.invokeFactory('Folder', 'f1')
         self.folder.f1.manage_permission('Modify view template', ('Manager',))
         self.failIf(self.folder.f1.canSetDefaultPage())
         actions = self.menu.getMenuItems(self.folder.f1, self.request)
-        self.failIf('_contextSetDefaultPage' in [a['extra']['id'] for a in actions])
+        self.failIf('contextSetDefaultPage' in
+                    [a['extra']['id'] for a in actions])
 
     def testSelectItemNotIncludedInNonStructuralFolder(self):
         self.folder.invokeFactory('Folder', 'f1')
         directlyProvides(self.folder.f1, INonStructuralFolder)
         actions = self.menu.getMenuItems(self.folder.f1, self.request)
-        self.failIf('_contextSetDefaultPage' in [a['extra']['id'] for a in actions])
+        self.failIf('contextSetDefaultPage' in
+                    [a['extra']['id'] for a in actions])
 
     def testDefaultPageSelectedAndOverridesLayout(self):
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder, self.request)
         selected = [a['extra']['id'] for a in actions if a['selected']]
-        self.assertEqual(selected, ['_contextDefaultPageDisplay'])
+        self.assertEqual(selected, ['contextDefaultPageDisplay'])
 
     def testDefaultPageCanBeChangedInContext(self):
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder, self.request)
-        self.failUnless('_contextChangeDefaultPage' in [a['extra']['id'] for a in actions])
+        self.failUnless('contextChangeDefaultPage' in
+                        [a['extra']['id'] for a in actions])
 
     def testDefaultPageCanBeChangedInFolder(self):
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
-        self.failUnless('_folderChangeDefaultPage' in [a['extra']['id'] for a in actions])
-        self.failIf('_contextChangeDefaultPage' in [a['extra']['id'] for a in actions])
+        self.failUnless('folderChangeDefaultPage' in
+                        [a['extra']['id'] for a in actions])
+        self.failIf('contextChangeDefaultPage' in
+                    [a['extra']['id'] for a in actions])
 
     # Headers/separators
 
     def testSeparatorsIncludedWhenViewingDefaultPageWithViews(self):
         fti = self.portal.portal_types['Document']
         documentViews = fti.view_methods + ('base_view',)
-        fti.manage_changeProperties(view_methods = documentViews)
+        fti.manage_changeProperties(view_methods=documentViews)
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
         ids = [a['extra']['id'] for a in actions]
-        self.failUnless('_folderHeader' in ids)
-        self.failUnless('_contextHeader' in ids)
+        self.failUnless('folderHeader' in ids)
+        self.failUnless('contextHeader' in ids)
 
     def testSeparatorsNotIncludedWhenViewingDefaultPageWithoutViews(self):
         self.folder.invokeFactory('Document', 'doc1')
@@ -158,35 +172,39 @@ class TestDisplayMenu(ptc.PloneTestCase):
         self.assertEqual(len(self.folder.doc1.getAvailableLayouts()), 1)
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
         ids = [a['extra']['id'] for a in actions]
-        self.failIf('_folderHeader' in ids)
-        self.failIf('_contextHeader' in ids)
+        self.failIf('folderHeader' in ids)
+        self.failIf('contextHeader' in ids)
 
     def testSeparatorsNotDisplayedWhenViewingFolder(self):
         fti = self.portal.portal_types['Document']
         documentViews = fti.view_methods + ('base_view',)
-        fti.manage_changeProperties(view_methods = documentViews)
+        fti.manage_changeProperties(view_methods=documentViews)
         self.folder.invokeFactory('Document', 'doc1')
         self.folder.setDefaultPage('doc1')
         actions = self.menu.getMenuItems(self.folder, self.request)
         ids = [a['extra']['id'] for a in actions]
-        self.failIf('_folderHeader' in ids)
-        self.failIf('_contextHeader' in ids)
+        self.failIf('folderHeader' in ids)
+        self.failIf('contextHeader' in ids)
 
     # Regressions
 
     def testDefaultPageTemplateTitle(self):
-        self.folder.setDefaultPage('sitemap') # Use a template in portal_skins
+        self.folder.setDefaultPage('sitemap')  # Use a template in portal_skins
         actions = self.menu.getMenuItems(self.folder, self.request)
-        changeAction = [x for x in actions if x['extra']['id'] == '_contextDefaultPageDisplay'][0]
+        changeAction = [x for x in actions if
+                        x['extra']['id'] == 'contextDefaultPageDisplay'][0]
         changeAction['title'].default
-        self.assertEquals(u"Sitemap", changeAction['title'].mapping['contentitem'])
+        self.assertEquals(u"Sitemap",
+                          changeAction['title'].mapping['contentitem'])
 
 
 class TestFactoriesMenu(ptc.PloneTestCase):
 
     def afterSetUp(self):
         self.folder.invokeFactory('Document', 'doc1')
-        self.menu = getUtility(IBrowserMenu, name='plone_contentmenu_factory', context=self.folder)
+        self.menu = getUtility(
+            IBrowserMenu, name='plone_contentmenu_factory',
+            context=self.folder)
         self.request = self.app.REQUEST
 
     def testMenuImplementsIBrowserMenu(self):
@@ -199,9 +217,35 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder, self.request)
         self.failUnless('image' in [a['extra']['id'] for a in actions])
 
+    def testAddViewExpressionUsedInMenu(self):
+        self.portal.portal_types['Image']._setPropValue(
+            'add_view_expr', 'string:custom_expr')
+        actions = self.menu.getMenuItems(self.folder, self.request)
+        self.failUnless('custom_expr' in [a['action'] for a in actions])
+        self.failUnless(
+            '%s/createObject?type_name=File' % self.folder.absolute_url()
+            in [a['action'] for a in actions])
+
+    def testFrontPageExpressionContext(self):
+        # If the expression context uses the front-page instead of the
+        # folder using the front-page, then the expression values will
+        # be incorrect.
+        self.portal.portal_types['Event']._setPropValue(
+            'add_view_expr', 'string:${folder_url}/+/addATEvent')
+        self.loginAsPortalOwner()
+        actions = self.menu.getMenuItems(
+            self.portal.events.aggregator, self.request)
+        self.failUnless(
+            'http://nohost/plone/events/+/addATEvent' in [a['action'] for a in actions])
+        self.failIf(
+            'http://nohost/plone/events/aggregator/+/addATEvent' in
+            [a['action'] for a in actions])
+
     def testTypeNameIsURLQuoted(self):
         actions = self.menu.getMenuItems(self.folder, self.request)
-        self.failUnless(self.folder.absolute_url() + '/createObject?type_name=News+Item' in [a['action'] for a in actions])
+        self.failUnless(
+            self.folder.absolute_url() + '/createObject?type_name=News+Item'
+            in [a['action'] for a in actions])
 
     def testMenuIncludesFactoriesOnNonFolderishContext(self):
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
@@ -221,6 +265,13 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.portal, self.request)
         self.assertEqual(len(actions), 0)
 
+        # set no types for folders and check the menu is not shown
+        folder_fti = self.portal.portal_types['Folder']
+        folder_fti.manage_changeProperties(
+            filter_content_types=True, allowed_content_types=[])
+        actions = self.menu.getMenuItems(self.folder, self.request)
+        self.assertEqual(len(actions), 0)
+
     def testConstrainTypes(self):
         constraints = ISelectableConstrainTypes(self.folder)
         constraints.setConstrainTypesMode(1)
@@ -229,11 +280,11 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder, self.request)
         self.assertEqual(len(actions), 2)
         self.assertEqual(actions[0]['extra']['id'], 'document')
-        self.assertEqual(actions[1]['extra']['id'], '_settings')
+        self.assertEqual(actions[1]['extra']['id'], 'settings')
 
     def testSettingsIncluded(self):
         actions = self.menu.getMenuItems(self.folder, self.request)
-        self.assertEqual(actions[-1]['extra']['id'], '_settings')
+        self.assertEqual(actions[-1]['extra']['id'], 'settings')
 
     def testSettingsNotIncludedWhereNotSupported(self):
         self.folder.manage_permission('Modify constrain types', ('Manager',))
@@ -248,8 +299,8 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder, self.request)
         self.failIf('image' in [a['extra']['id'] for a in actions])
         self.failUnless('document' in [a['extra']['id'] for a in actions])
-        self.failUnless('_more' in [a['extra']['id'] for a in actions])
-        self.failUnless('_settings' in [a['extra']['id'] for a in actions])
+        self.failUnless('more' in [a['extra']['id'] for a in actions])
+        self.failUnless('settings' in [a['extra']['id'] for a in actions])
 
     def testMoreNotIncludedWhenNotNecessary(self):
         constraints = ISelectableConstrainTypes(self.folder)
@@ -259,16 +310,9 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder, self.request)
         self.assertEqual(len(actions), 2)
         self.assertEqual(actions[0]['extra']['id'], 'document')
-        self.assertEqual(actions[1]['extra']['id'], '_settings')
+        self.assertEqual(actions[1]['extra']['id'], 'settings')
 
     def testNonStructualFolderShowsParent(self):
-        # Cope with an unfortunate side-effect of running all plone.* tests at
-        # once. Some tests setup the Archetypes examples types and don't use
-        # their own layer for it :(
-        for t in ('ATBIFolder', 'ComplexType', 'DDocument'):
-            if t in self.portal.portal_types.keys():
-                self.portal.portal_types[t].global_allow = False
-
         self.folder.invokeFactory('Folder', 'folder1')
         directlyProvides(self.folder.folder1, INonStructuralFolder)
         constraints = ISelectableConstrainTypes(self.folder.folder1)
@@ -276,14 +320,27 @@ class TestFactoriesMenu(ptc.PloneTestCase):
         constraints.setLocallyAllowedTypes(('Document',))
         constraints.setImmediatelyAddableTypes(('Document',))
         actions = self.menu.getMenuItems(self.folder.folder1, self.request)
-        self.failUnless('event' in actions[0]['extra']['id'])
+        action_ids = [a['extra']['id'] for a in actions]
+        self.failUnless('event' in action_ids)
+
+    def testImgConditionalOnTypeIcon(self):
+        """The <img> element should not render if the content type has
+        no icon expression"""
+        folder_fti = self.portal.portal_types['Folder']
+        folder_fti.manage_changeProperties(icon_expr='')
+        for item in self.menu.getMenuItems(self.folder, self.request):
+            if item['id'] == folder_fti.getId():
+                break
+        self.failIf(item['icon'])
 
 
 class TestWorkflowMenu(ptc.PloneTestCase):
 
     def afterSetUp(self):
         self.folder.invokeFactory('Document', 'doc1')
-        self.menu = getUtility(IBrowserMenu, name='plone_contentmenu_workflow', context=self.folder)
+        self.menu = getUtility(
+            IBrowserMenu, name='plone_contentmenu_workflow',
+            context=self.folder)
         self.request = self.app.REQUEST
 
     def testMenuImplementsIBrowserMenu(self):
@@ -296,8 +353,9 @@ class TestWorkflowMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
         self.failUnless('workflow-transition-submit' in
                         [a['extra']['id'] for a in actions])
-        self.failUnless('http://nohost/plone/Members/test_user_1_/doc1/content_status_modify?workflow_action=submit' in
-                        [a['action'] for a in actions])
+        self.failUnless(('http://nohost/plone/Members/test_user_1_/doc1/'
+                         'content_status_modify?workflow_action=submit')
+                        in [a['action'] for a in actions])
 
         # Let us try that again but with an empty url action, like is
         # usual in older workflows, and which is nice to keep
@@ -309,7 +367,8 @@ class TestWorkflowMenu(ptc.PloneTestCase):
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
         self.failUnless('workflow-transition-submit' in
                         [a['extra']['id'] for a in actions])
-        self.failUnless('http://nohost/plone/Members/test_user_1_/doc1/content_status_modify?workflow_action=submit' in
+        self.failUnless(('http://nohost/plone/Members/test_user_1_/doc1/'
+                         'content_status_modify?workflow_action=submit') in
                         [a['action'] for a in actions])
 
     def testNoTransitions(self):
@@ -333,9 +392,15 @@ class TestWorkflowMenu(ptc.PloneTestCase):
 
     def testPolicyIncludedIfCMFPWIsInstalled(self):
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
-        url = self.folder.doc1.absolute_url() + '/placeful_workflow_configuration'
+        url = self.folder.doc1.absolute_url(
+            ) + '/placeful_workflow_configuration'
         self.failIf(url in [a['action'] for a in actions])
         self.portal.portal_quickinstaller.installProduct('CMFPlacefulWorkflow')
+
+        # item needs permission
+        actions = self.menu.getMenuItems(self.folder.doc1, self.request)
+        self.failIf(url in [a['action'] for a in actions])
+        self.loginAsPortalOwner()
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
         self.failUnless(url in [a['action'] for a in actions])
 
@@ -343,80 +408,115 @@ class TestWorkflowMenu(ptc.PloneTestCase):
 class TestContentMenu(ptc.PloneTestCase):
 
     def afterSetUp(self):
-        self.menu = getUtility(IBrowserMenu, name='plone_contentmenu', context=self.folder)
+        self.menu = getUtility(
+            IBrowserMenu, name='plone_contentmenu', context=self.folder)
         self.request = self.app.REQUEST
 
     # Actions sub-menu
 
     def testActionsSubMenuIncluded(self):
         items = self.menu.getMenuItems(self.folder, self.request)
-        actionsMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-actions'][0]
-        self.assertEqual(actionsMenuItem['action'], self.folder.absolute_url() + '/folder_contents')
+        actionsMenuItem = [i for i in items if
+                           i['extra']['id'] == 'plone-contentmenu-actions'][0]
+        self.assertEqual(actionsMenuItem['action'],
+                         self.folder.absolute_url() + '/folder_contents')
         self.failUnless(len(actionsMenuItem['submenu']) > 0)
 
     # Display sub-menu
 
     def testDisplayMenuIncluded(self):
         items = self.menu.getMenuItems(self.folder, self.request)
-        displayMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-display'][0]
-        self.assertEqual(displayMenuItem['action'], self.folder.absolute_url() + '/select_default_view')
+        displayMenuItem = [i for i in items if
+                           i['extra']['id'] == 'plone-contentmenu-display'][0]
+        self.assertEqual(displayMenuItem['action'],
+                         self.folder.absolute_url() + '/select_default_view')
         self.failUnless(len(displayMenuItem['submenu']) > 0)
 
     def testDisplayMenuNotIncludedIfContextDoesNotSupportBrowserDefault(self):
-        # We need to create an object that does not have IBrowserDefault enabled
+        # We need to create an object that does not have
+        # IBrowserDefault enabled
         _createObjectByType('ATListCriterion', self.folder, 'c1')
         items = self.menu.getMenuItems(self.folder.c1, self.request)
-        self.assertEqual([i for i in items if i['extra']['id'] == 'plone-contentmenu-display'], [])
+        self.assertEqual([i for i in items if
+                          i['extra']['id'] == 'plone-contentmenu-display'], [])
 
-    def testDisplayMenuShowFolderDefaultPageWhenContextDoesNotSupportSelectableBrowserDefault(self):
-        # We need to create an object that is not ISelectableBrowserDefault aware
+    def testWhenContextDoesNotSupportSelectableBrowserDefault(self):
+        """Display Menu Show Folder Default Page When Context Does Not
+        Support Selectable Browser Default"""
+        # We need to create an object that is not
+        # ISelectableBrowserDefault aware
         _createObjectByType('ATListCriterion', self.folder, 'c1')
         self.folder.c1.setTitle('Foo')
         self.folder.setDefaultPage('c1')
         items = self.menu.getMenuItems(self.folder.c1, self.request)
-        displayMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-display'][0]
-        selected =  [a for a in displayMenuItem['submenu'] if a['selected']][0]
+        displayMenuItem = [i for i in items if
+                           i['extra']['id'] == 'plone-contentmenu-display'][0]
+        selected = [a for a in displayMenuItem['submenu']
+                    if a['selected']][0]
         self.assertEqual(u'Foo', selected['title'].mapping['contentitem'])
 
     def testDisplayMenuNotIncludedIfNoActionsAvailable(self):
         self.folder.invokeFactory('Document', 'doc1')
         items = self.menu.getMenuItems(self.folder.doc1, self.request)
-        self.assertEqual([i for i in items if i['extra']['id'] == 'plone-contentmenu-display'], [])
+        self.assertEqual([i for i in items if
+                          i['extra']['id'] == 'plone-contentmenu-display'], [])
 
     def testDisplayMenuDisabledIfIndexHtmlInFolder(self):
         self.folder.invokeFactory('Document', 'index_html')
         items = self.menu.getMenuItems(self.folder, self.request)
-        displayMenuItems = [i for i in items if i['extra']['id'] == 'plone-contentmenu-display']
+        displayMenuItems = [i for i in items if
+                            i['extra']['id'] == 'plone-contentmenu-display']
         self.assertEqual(len(displayMenuItems), 0)
 
     def testDisplayMenuDisabledIfIndexHtmlInFolderAndContextIsIndexHtml(self):
         self.folder.invokeFactory('Document', 'index_html')
         items = self.menu.getMenuItems(self.folder.index_html, self.request)
-        displayMenuItems = [i for i in items if i['extra']['id'] == 'plone-contentmenu-display']
+        displayMenuItems = [i for i in items if
+                            i['extra']['id'] == 'plone-contentmenu-display']
         self.assertEqual(len(displayMenuItems), 0)
 
+    def testDisplayMenuAddPrefixFolderForContainerPart(self):
+        prefix = 'folder-'
+        self.folder.invokeFactory('Folder', 'subfolder1')
+        self.folder.setDefaultPage('subfolder1')
+        items = self.menu.getMenuItems(self.folder.subfolder1, self.request)
+        displayMenuItems = [i for i in items if
+                            i['extra']['id'] == 'plone-contentmenu-display'][0]
+        extras = [i['extra'] for i in displayMenuItems['submenu']]
+        for extra in extras[1:]:
+            if not extra['separator'] is None: break
+            else:
+                self.assertEqual(extra['id'][0:len(prefix)], prefix)
+
     # Add sub-menu
 
     def testAddMenuIncluded(self):
         items = self.menu.getMenuItems(self.folder, self.request)
-        factoriesMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-factories'][0]
-        self.assertEqual(factoriesMenuItem['action'], self.folder.absolute_url() + '/folder_factories')
+        factoriesMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-factories'][0]
+        self.assertEqual(factoriesMenuItem['action'],
+                         self.folder.absolute_url() + '/folder_factories')
         self.failUnless(len(factoriesMenuItem['submenu']) > 0)
-        self.assertEqual(factoriesMenuItem['extra']['hideChildren'], False)
 
     def testAddMenuNotIncludedIfNothingToAdd(self):
         self.logout()
         items = self.menu.getMenuItems(self.folder, self.request)
-        self.assertEqual([i for i in items if i['extra']['id'] == 'plone-contentmenu-factories'], [])
+        self.assertEqual(
+            [i for i in items if
+             i['extra']['id'] == 'plone-contentmenu-factories'], [])
 
     def testAddMenuWithNothingToAddButWithAvailableConstrainSettings(self):
         self.folder.setConstrainTypesMode(1)
         self.folder.setLocallyAllowedTypes(())
         self.folder.setImmediatelyAddableTypes(())
         items = self.menu.getMenuItems(self.folder, self.request)
-        factoriesMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-factories'][0]
+        factoriesMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-factories'][0]
         self.assertEqual(len(factoriesMenuItem['submenu']), 1)
-        self.assertEqual(factoriesMenuItem['submenu'][0]['extra']['id'], '_settings')
+        self.assertEqual(factoriesMenuItem['submenu'][0]['extra']['id'],
+                         'settings')
 
     def testAddMenuWithNothingToAddButWithAvailableMorePage(self):
         self.folder.setConstrainTypesMode(1)
@@ -424,41 +524,55 @@ class TestContentMenu(ptc.PloneTestCase):
         self.folder.setImmediatelyAddableTypes(())
         self.folder.manage_permission('Modify constrain types', ('Manager',))
         items = self.menu.getMenuItems(self.folder, self.request)
-        factoriesMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-factories'][0]
+        factoriesMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-factories'][0]
         self.assertEqual(len(factoriesMenuItem['submenu']), 1)
-        self.assertEqual(factoriesMenuItem['submenu'][0]['extra']['id'], '_more')
+        self.assertEqual(factoriesMenuItem['submenu'][0]['extra']['id'],
+                         'more')
 
     def testAddMenuRelativeToNonStructuralFolder(self):
         self.folder.invokeFactory('Folder', 'f1')
         directlyProvides(self.folder.f1, INonStructuralFolder)
         items = self.menu.getMenuItems(self.folder.f1, self.request)
-        factoriesMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-factories']
+        factoriesMenuItem = [i for i in items if
+                             i['extra']['id'] == 'plone-contentmenu-factories']
         self.failIf(factoriesMenuItem)
 
-    def testAddMenuWithSingleItemCollapses(self):
+    def testAddMenuWithAddViewExpr(self):
         # we need a dummy to test this - should test that if the item does not
         # support constrain types and there is
         self.folder.setConstrainTypesMode(1)
         self.folder.setLocallyAllowedTypes(('Document',))
         self.folder.setImmediatelyAddableTypes(('Document',))
         self.folder.manage_permission('Modify constrain types', ('Manager',))
+        self.portal.portal_types['Document']._setPropValue(
+            'add_view_expr', 'string:custom_expr')
         items = self.menu.getMenuItems(self.folder, self.request)
-        factoriesMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-factories'][0]
-        self.assertEqual(factoriesMenuItem['action'], self.folder.absolute_url() + '/createObject?type_name=Document')
-        self.assertEqual(factoriesMenuItem['extra']['hideChildren'], True)
+        factoriesMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-factories'][0]
+        self.assertEqual(factoriesMenuItem['submenu'][0]['action'],
+                         'custom_expr')
 
     # Workflow sub-menu
 
     def testWorkflowMenuIncluded(self):
         items = self.menu.getMenuItems(self.folder, self.request)
-        workflowMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-workflow'][0]
-        self.assertEqual(workflowMenuItem['action'], self.folder.absolute_url() + '/content_status_history')
+        workflowMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-workflow'][0]
+        self.assertEqual(
+            workflowMenuItem['action'],
+            self.folder.absolute_url() + '/content_status_history')
         self.failUnless(len(workflowMenuItem['submenu']) > 0)
 
     def testWorkflowMenuWithNoTransitionsDisabled(self):
         self.logout()
         items = self.menu.getMenuItems(self.folder, self.request)
-        workflowMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-workflow'][0]
+        workflowMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-workflow'][0]
         self.assertEqual(workflowMenuItem['action'], '')
 
     # XXX: Unable to write a proper test so far
@@ -473,7 +587,9 @@ class TestContentMenu(ptc.PloneTestCase):
         wf.transitions['publish'].guard.expr = Expression('python: False')
 
         items = self.menu.getMenuItems(self.folder, self.request)
-        workflowMenuItem = [i for i in items if i['extra']['id'] == 'plone-contentmenu-workflow'][0]
+        workflowMenuItem = [
+            i for i in items if
+            i['extra']['id'] == 'plone-contentmenu-workflow'][0]
 
         # A regular user doesn't see any actions
         self.failUnless(workflowMenuItem['action'] == '')
@@ -485,13 +601,18 @@ class TestContentMenu(ptc.PloneTestCase):
         self.portal.portal_workflow.setChainForPortalTypes(('Document',), ())
         self.folder.invokeFactory('Document', 'doc1')
         actions = self.menu.getMenuItems(self.folder.doc1, self.request)
-        self.failIf('plone_contentmenu_workflow' in [a['extra']['id'] for a in actions])
+        self.failIf('plone_contentmenu_workflow' in
+                    [a['extra']['id'] for a in actions])
 
 
 class TestDisplayViewsMenu(PlacelessSetup, unittest.TestCase):
 
     def setUp(self):
-        from Products.Five import zcml
+        # BBB for Zope 2.12
+        try:
+            from Zope2.App import zcml
+        except ImportError:
+            from Products.Five import zcml
         import Products.Five
         import plone.app.contentmenu
         zcml.load_config("meta.zcml", Products.Five)
@@ -507,7 +628,7 @@ class TestDisplayViewsMenu(PlacelessSetup, unittest.TestCase):
 
     def testInterface(self):
         """A DisplayViewsMenu implements an extended interface"""
-        from interfaces import IDisplayViewsMenu
+        from plone.app.contentmenu.interfaces import IDisplayViewsMenu
         self.assertTrue(IDisplayViewsMenu.providedBy(self.menu))
 
     def testSimpleAction(self):
